'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports['default'] = InfiniteCanvas;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _clearCanvas = require('clear-canvas');

var _clearCanvas2 = _interopRequireDefault(_clearCanvas);

var _drawToCanvas = require('draw-to-canvas');

var _drawToCanvas2 = _interopRequireDefault(_drawToCanvas);

var _resizeCanvas = require('resize-canvas');

var _resizeCanvas2 = _interopRequireDefault(_resizeCanvas);

var _copyCanvas = require('copy-canvas');

var _copyCanvas2 = _interopRequireDefault(_copyCanvas);

var _glVec2Add = require('gl-vec2/add');

var _glVec2Add2 = _interopRequireDefault(_glVec2Add);

var _glVec2Max = require('gl-vec2/max');

var _glVec2Max2 = _interopRequireDefault(_glVec2Max);

var _glVec2Min = require('gl-vec2/min');

var _glVec2Min2 = _interopRequireDefault(_glVec2Min);

var _glVec2Subtract = require('gl-vec2/subtract');

var _glVec2Subtract2 = _interopRequireDefault(_glVec2Subtract);

// utility functions
var getDimensions = function getDimensions(canvas) {
  return [canvas.width, canvas.height];
};
var abs = function abs(arr) {
  return arr.map(Math.abs);
};

// calculate where to scale from. If x or y is positive, scale from 0
// otherwise scale from bottom or right
function getResizeFrom(canvas, originDiff) {
  var resizeFrom = getDimensions(canvas);
  if (originDiff[0] <= 0) resizeFrom[0] = 0;
  if (originDiff[1] <= 0) resizeFrom[1] = 0;
  return resizeFrom;
}

function InfiniteCanvas(canvas) {

  var canvasContext = canvas.getContext('2d');
  var buffer = (0, _copyCanvas2['default'])(canvas);
  var bufferContext = buffer.getContext('2d');

  // private properties
  var origin = [0, 0];
  var originMax = [0, 0];
  var originMin = [0, 0];
  var originDraw = [0, 0];

  // set the absolute [x, y] origin position of the buffer canvas on
  // the main canvas to pan around
  function setOrigin(newOrigin) {

    // update origins
    var originDiff = (0, _glVec2Subtract2['default'])([], newOrigin, origin);
    origin = newOrigin;
    originMax = (0, _glVec2Max2['default'])([], origin, originMax);
    originMin = (0, _glVec2Min2['default'])([], origin, originMin);

    // get dimensions
    var canvasDimensions = getDimensions(canvas);
    var bufferDimensions = getDimensions(buffer);
    var maxDimensions = (0, _glVec2Add2['default'])([], canvasDimensions, (0, _glVec2Add2['default'])([], originMax, abs(originMin)));

    // resize
    var maxDimensionsDiff = (0, _glVec2Subtract2['default'])([], maxDimensions, bufferDimensions);
    (0, _resizeCanvas2['default'])({
      canvas: buffer,
      diff: maxDimensionsDiff,
      from: getResizeFrom(buffer, originDiff)
    });

    // redraw
    originDraw = (0, _glVec2Min2['default'])([], [0, 0], (0, _glVec2Add2['default'])([], originDraw, originDiff));
    refresh();
  }

  // pan with relative [x, y] values
  function move(originDiff) {
    setOrigin((0, _glVec2Add2['default'])([], origin, originDiff));
  }

  // copy whatever was drawn on the canvas to the buffer
  function commitToBuffer() {
    (0, _drawToCanvas2['default'])(canvas, bufferContext, origin);
  }

  // redraw to the canvas from the buffer
  function refresh() {

    // erase canvas
    (0, _clearCanvas2['default'])(canvasContext);

    // draw to canvas
    (0, _drawToCanvas2['default'])(buffer, canvasContext, originDraw);
  }

  return Object.freeze({
    canvas: canvas,
    buffer: buffer,
    canvasContext: canvasContext,
    bufferContext: bufferContext,
    getOrigin: function getOrigin() {
      return origin;
    },
    setOrigin: setOrigin,
    move: move,
    commitToBuffer: commitToBuffer,
    refresh: refresh
  });
}

module.exports = exports['default'];

